package no.ntnu.idatt1002.demo.data.recipes;

import java.util.ArrayList;
import java.util.List;

/**
 * The IngredientsAtHand class contains a collection of Ingredient objects that are currently available to the user.
 * Only one instance of each ingredient type may exist in the collection. If an ingredient is already present in the
 * collection, the old will be replaced by the new.
 *
 * @author hannesofie
 */
public class IngredientsAtHand {

    private final List<Ingredient> ingredientsAtHand = new ArrayList<>();

    /**
     * The method returns the collection of ingredients at hand as an arraylist of ingredient objects.
     * @return The collection of ingredients at hand to the user.
     */
    public List<Ingredient> getIngredientsAtHand() {
        return ingredientsAtHand;
    }

    /**
     * The method takes in an ingredient object and adds it to the collection of ingredients at hand.
     * @param ingredient The ingredient object to add to the collection of ingredients at hand.
     */
    public void addIngredient(Ingredient ingredient) {
        this.ingredientsAtHand.add(ingredient);
    }

    /**
     * Returns null if no ingredient of the requested type is found in the collection.
     * @param ingredientType What type of food the ingredient is.
     * @return The ingredient of the specified type found among the ingredients at hand, null otherwise.
     */
    public Ingredient getIngredient(FoodItem ingredientType) {
        if(ingredientType == null) return null;
        return this.getIngredientsAtHand().stream()
                .filter((ingredient) -> ingredient.getFoodType() == ingredientType)
                .findFirst().orElse(null);
    }

    /**
     * The method takes in three parameters. The method first checks if the Ingredient is at hand in the first place.
     * If it is, the old amount and unit of this ingredient are replaced by the provided amount and unit if they
     * differ. If not, the ingredient is left as is. If the ingredient is not in the collection,
     * @param ingredientType What type of food the ingredient is.
     * @param amount The amount of the ingredient.
     * @return True if Ingredient is successfully altered or added, false if not.
     */
    public boolean alterIngredient(FoodItem ingredientType, double amount, MeasuringUnit unit) {
        //TODO: Consider handling exceptions differently.
        if(ingredientsAtHand.stream().anyMatch((ingredient) -> ingredient.getFoodType() == ingredientType)) {
            try {
                getIngredient(ingredientType).setAmount(amount);
                getIngredient(ingredientType).setUnit(unit);

            } catch (IllegalArgumentException e) {
                return false;
            }
        } else {
            try {
                addIngredient(new Ingredient(ingredientType, amount, unit));
            } catch (IllegalArgumentException e) {
                return false;
            }
        }
        return true;
    }

    /**
     * The method takes in a value of the FoodItem enum as a parameter and removes it from the collection of
     * ingredients at hand if it exists and returns true. If no ingredient of the given type was found in the
     * collection, false is returned.
     * @param ingredientType What type of food the ingredient is.
     * @return True if the ingredient was found among the ingredients at hand and removed, false otherwise.
     */
    public boolean removeIngredient(FoodItem ingredientType) {
        return ingredientsAtHand.removeIf((ingredient) -> ingredient.getFoodType() == ingredientType);
    }

}
